@@ -17,8 +17,8 @@ unsigned char *c1943_bgscrolly;
 static int chon,objon,sc1on,sc2on;
 static int flipscreen;
 
-static struct osd_bitmap *sc2bitmap;
-static struct osd_bitmap *sc1bitmap;
+static struct mame_bitmap *sc2bitmap;
+static struct mame_bitmap *sc1bitmap;
 static unsigned char sc2map[9][8][2];
 static unsigned char sc1map[9][9][2];
 
@@ -38,7 +38,7 @@ static unsigned char sc1map[9][9][2];
   bit 0 -- 2.2kohm resistor  -- RED/GREEN/BLUE
 
 ***************************************************************************/
-void c1943_vh_convert_color_prom(unsigned char *palette, unsigned char *colortable,const unsigned char *color_prom)
+PALETTE_INIT( 1943 )
 {
 	int i;
 	#define TOTAL_COLORS(gfxn) (Machine->gfx[gfxn]->total_colors * Machine->gfx[gfxn]->color_granularity)
@@ -47,29 +47,29 @@ void c1943_vh_convert_color_prom(unsigned char *palette, unsigned char *colortab
 
 	for (i = 0;i < Machine->drv->total_colors;i++)
 	{
-		int bit0,bit1,bit2,bit3;
+		int bit0,bit1,bit2,bit3,r,g,b;
 
 
-		bit0 = (color_prom[0] >> 0) & 0x01;
-		bit1 = (color_prom[0] >> 1) & 0x01;
-		bit2 = (color_prom[0] >> 2) & 0x01;
-		bit3 = (color_prom[0] >> 3) & 0x01;
-		*(palette++) = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
-		bit0 = (color_prom[Machine->drv->total_colors] >> 0) & 0x01;
-		bit1 = (color_prom[Machine->drv->total_colors] >> 1) & 0x01;
-		bit2 = (color_prom[Machine->drv->total_colors] >> 2) & 0x01;
-		bit3 = (color_prom[Machine->drv->total_colors] >> 3) & 0x01;
-		*(palette++) = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
-		bit0 = (color_prom[2*Machine->drv->total_colors] >> 0) & 0x01;
-		bit1 = (color_prom[2*Machine->drv->total_colors] >> 1) & 0x01;
-		bit2 = (color_prom[2*Machine->drv->total_colors] >> 2) & 0x01;
-		bit3 = (color_prom[2*Machine->drv->total_colors] >> 3) & 0x01;
-		*(palette++) = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
+		bit0 = (color_prom[i] >> 0) & 0x01;
+		bit1 = (color_prom[i] >> 1) & 0x01;
+		bit2 = (color_prom[i] >> 2) & 0x01;
+		bit3 = (color_prom[i] >> 3) & 0x01;
+		r = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
+		bit0 = (color_prom[i + Machine->drv->total_colors] >> 0) & 0x01;
+		bit1 = (color_prom[i + Machine->drv->total_colors] >> 1) & 0x01;
+		bit2 = (color_prom[i + Machine->drv->total_colors] >> 2) & 0x01;
+		bit3 = (color_prom[i + Machine->drv->total_colors] >> 3) & 0x01;
+		g = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
+		bit0 = (color_prom[i + 2*Machine->drv->total_colors] >> 0) & 0x01;
+		bit1 = (color_prom[i + 2*Machine->drv->total_colors] >> 1) & 0x01;
+		bit2 = (color_prom[i + 2*Machine->drv->total_colors] >> 2) & 0x01;
+		bit3 = (color_prom[i + 2*Machine->drv->total_colors] >> 3) & 0x01;
+		b = 0x0e * bit0 + 0x1f * bit1 + 0x43 * bit2 + 0x8f * bit3;
 
-		color_prom++;
+		palette_set_color(i,r,g,b);
 	}
 
-	color_prom += 2*Machine->drv->total_colors;
+	color_prom += 3*Machine->drv->total_colors;
 	/* color_prom now points to the beginning of the lookup table */
 
 	/* characters use colors 64-79 */
@@ -110,12 +110,34 @@ void c1943_vh_convert_color_prom(unsigned char *palette, unsigned char *colortab
 
 
 
-void c1943_c804_w(int offset,int data)
+VIDEO_START( 1943 )
+{
+	if ((sc2bitmap = auto_bitmap_alloc(9*32,8*32)) == 0)
+		return 1;
+
+	if ((sc1bitmap = auto_bitmap_alloc(9*32,9*32)) == 0)
+		return 1;
+
+	if (video_start_generic())
+		return 1;
+
+	memset (sc2map, 0xff, sizeof (sc2map));
+	memset (sc1map, 0xff, sizeof (sc1map));
+
+	return 0;
+}
+
+
+
+WRITE8_HANDLER( c1943_c804_w )
 {
 	int bankaddress;
+	unsigned char *RAM = memory_region(REGION_CPU1);
 
 
-	/* bits 0 and 1 are for coin counters - we ignore them */
+	/* bits 0 and 1 are coin counters */
+	coin_counter_w(0,data & 1);
+	coin_counter_w(1,data & 2);
 
 	/* bits 2, 3 and 4 select the ROM bank */
 	bankaddress = 0x10000 + (data & 0x1c) * 0x1000;
@@ -127,7 +149,7 @@ void c1943_c804_w(int offset,int data)
 	if (flipscreen != (data & 0x40))
 	{
 		flipscreen = data & 0x40;
-/*		memset(dirtybuffer,1,c1942_backgroundram_size);*/
+/*		memset(dirtybuffer,1,c1942_backgroundram_size); */
 	}
 
 	/* bit 7 enables characters */
@@ -136,7 +158,7 @@ void c1943_c804_w(int offset,int data)
 
 
 
-void c1943_d806_w(int offset,int data)
+WRITE8_HANDLER( c1943_d806_w )
 {
 	/* bit 4 enables bg 1 */
 	sc1on = data & 0x10;
@@ -149,48 +171,15 @@ void c1943_d806_w(int offset,int data)
 }
 
 
-int c1943_vh_start(void)
-{
-	if ((sc2bitmap = osd_create_bitmap(8*32,9*32)) == 0)
-		return 1;
-	
-	if ((sc1bitmap = osd_create_bitmap(9*32,9*32)) == 0)
-	{
-		osd_free_bitmap(sc2bitmap);
-		return 1;
-	}
-	
-	if (generic_vh_start() == 1)
-	{
-		osd_free_bitmap(sc2bitmap);
-		osd_free_bitmap(sc1bitmap);
-		return 1;
-	}
-	
-	memset (sc2map, 0xff, sizeof (sc2map));
-	memset (sc1map, 0xff, sizeof (sc1map));
-
-	return 0;
-}
-
-
-void c1943_vh_stop(void)
-{
-	osd_free_bitmap(sc2bitmap);
-	osd_free_bitmap(sc1bitmap);
-}
-
-
-
 
 /***************************************************************************
 
-  Draw the game screen in the given osd_bitmap.
+  Draw the game screen in the given mame_bitmap.
   Do NOT call osd_update_display() from this function, it will be called by
   the main emulation engine.
 
 ***************************************************************************/
-void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
+VIDEO_UPDATE( 1943 )
 {
 	int offs,sx,sy;
 	int bg_scrolly, bg_scrollx;
@@ -200,7 +189,7 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 /* TODO: support flipscreen */
 	if (sc2on)
 	{
-		p=Machine->memory_region[3]+0x8000;
+		p=memory_region(REGION_GFX5)+0x8000;
 		bg_scrolly = c1943_bgscrolly[0] + 256 * c1943_bgscrolly[1];
 		offs = 16 * ((bg_scrolly>>5)+8);
 
@@ -221,7 +210,7 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 
 				tile=p[offset];
 				attr=p[offset+1];
-				
+
 				if (tile != map[0] || attr != map[1])
 				{
 					map[0] = tile;
@@ -229,8 +218,8 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 					drawgfx(sc2bitmap,Machine->gfx[2],
 							tile,
 							(attr & 0x3c) >> 2,
-							attr&0x80, attr&0x40,
-							sx*32, ty*32,
+							attr&0x40, attr&0x80,
+							(8-ty)*32, sx*32,
 							0,
 							TRANSPARENCY_NONE,0);
 				}
@@ -239,15 +228,15 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 			offs-=0x10;
 		}
 
-		xscroll = 0;
-		yscroll = -(top*32+32-bg_scrolly);
+		xscroll = (top*32-bg_scrolly);
+		yscroll = 0;
 		copyscrollbitmap(bitmap,sc2bitmap,
 			1,&xscroll,
 			1,&yscroll,
-			&Machine->drv->visible_area,
+			cliprect,
 			TRANSPARENCY_NONE,0);
 	}
-	else fillbitmap(bitmap,Machine->pens[0],&Machine->drv->visible_area);
+	else fillbitmap(bitmap,get_black_pen(),cliprect);
 
 
 	if (objon)
@@ -262,8 +251,8 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 			if (color == 0x0a || color == 0x0b)	/* the priority is actually selected by */
 												/* bit 3 of BMPROM.07 */
 			{
-				sx = spriteram[offs + 2];
-				sy = 240 - spriteram[offs + 3] + ((spriteram[offs + 1] & 0x10) << 4);
+				sx = spriteram[offs + 3] - ((spriteram[offs + 1] & 0x10) << 4);
+				sy = spriteram[offs + 2];
 				if (flipscreen)
 				{
 					sx = 240 - sx;
@@ -275,7 +264,7 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 						color,
 						flipscreen,flipscreen,
 						sx,sy,
-						&Machine->drv->visible_area,TRANSPARENCY_PEN,0);
+						cliprect,TRANSPARENCY_PEN,0);
 			}
 		}
 	}
@@ -284,7 +273,7 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 /* TODO: support flipscreen */
 	if (sc1on)
 	{
-		p=Machine->memory_region[3];
+		p=memory_region(REGION_GFX5);
 
 		bg_scrolly = c1943_scrolly[0] + 256 * c1943_scrolly[1];
 		bg_scrollx = c1943_scrollx[0];
@@ -301,7 +290,7 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 		{
 			int ty = (sy + top) % 9;
 			offs &= 0x7fff; /* Enforce limits (for top of scroll) */
-			
+
 			for (sx = 0;sx < 9;sx++)
 			{
 				int tile, attr, offset;
@@ -311,7 +300,7 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 
 				tile=p[offset];
 				attr=p[offset+1];
-				
+
 				if (tile != map[0] || attr != map[1])
 				{
 					map[0] = tile;
@@ -320,8 +309,8 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 					drawgfx(sc1bitmap,Machine->gfx[1],
 							tile,
 							(attr & 0x3c) >> 2,
-							attr & 0x80,attr & 0x40,
-							tx*32, ty*32,
+							attr & 0x40,attr & 0x80,
+							(8-ty)*32, tx*32,
 							0,
 							TRANSPARENCY_NONE,0);
 				}
@@ -329,12 +318,12 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 			offs-=0x10;
 		}
 
-		xscroll = -(left*32+bg_scrollx);
-		yscroll = -(top*32+32-bg_scrolly);
+		xscroll = (top*32-bg_scrolly);
+		yscroll = -(left*32+bg_scrollx);
 		copyscrollbitmap(bitmap,sc1bitmap,
 			1,&xscroll,
 			1,&yscroll,
-			&Machine->drv->visible_area,
+			cliprect,
 			TRANSPARENCY_COLOR,0);
 	}
 
@@ -351,8 +340,8 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 			if (color != 0x0a && color != 0x0b)	/* the priority is actually selected by */
 												/* bit 3 of BMPROM.07 */
 			{
-				sx = spriteram[offs + 2];
-				sy = 240 - spriteram[offs + 3] + ((spriteram[offs + 1] & 0x10) << 4);
+				sx = spriteram[offs + 3] - ((spriteram[offs + 1] & 0x10) << 4);
+				sy = spriteram[offs + 2];
 				if (flipscreen)
 				{
 					sx = 240 - sx;
@@ -364,7 +353,7 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 						color,
 						flipscreen,flipscreen,
 						sx,sy,
-						&Machine->drv->visible_area,TRANSPARENCY_PEN,0);
+						cliprect,TRANSPARENCY_PEN,0);
 			}
 		}
 	}
@@ -375,31 +364,20 @@ void c1943_vh_screenrefresh(struct osd_bitmap *bitmap)
 		/* draw the frontmost playfield. They are characters, but draw them as sprites */
 		for (offs = videoram_size - 1;offs >= 0;offs--)
 		{
-			int code;
-
-
-			code = videoram[offs] + ((colorram[offs] & 0xe0) << 3);
-
-			if (code != 0x24 || colorram[offs] != 0)	/* don't draw spaces */
+			sx = offs % 32;
+			sy = offs / 32;
+			if (flipscreen)
 			{
-				int sx,sy;
-
-
-				sx = offs / 32;
-				sy = 31 - offs % 32;
-				if (flipscreen)
-				{
-					sx = 31 - sx;
-					sy = 31 - sy;
-				}
-
-				drawgfx(bitmap,Machine->gfx[0],
-						code,
-						colorram[offs] & 0x1f,
-						flipscreen,flipscreen,
-						8*sx,8*sy,
-						&Machine->drv->visible_area,TRANSPARENCY_COLOR,79);
+				sx = 31 - sx;
+				sy = 31 - sy;
 			}
+
+			drawgfx(bitmap,Machine->gfx[0],
+					videoram[offs] + ((colorram[offs] & 0xe0) << 3),
+					colorram[offs] & 0x1f,
+					flipscreen,flipscreen,
+					8*sx,8*sy,
+					cliprect,TRANSPARENCY_COLOR,79);
 		}
 	}
 }
