@@ -9,7 +9,9 @@
 #include "driver.h"
 #include "vidhrdw/generic.h"
 
+static int gfxbank;
 
+static struct tilemap *bg_tilemap;
 
 /***************************************************************************
 
@@ -32,7 +34,7 @@
   bit 0 -- 1  kohm resistor  -- RED
 
 ***************************************************************************/
-void champbas_vh_convert_color_prom(unsigned char *palette, unsigned char *colortable,const unsigned char *color_prom)
+PALETTE_INIT( champbas )
 {
 	int i;
 	#define TOTAL_COLORS(gfxn) (Machine->gfx[gfxn]->total_colors * Machine->gfx[gfxn]->color_granularity)
@@ -41,87 +43,120 @@ void champbas_vh_convert_color_prom(unsigned char *palette, unsigned char *color
 
 	for (i = 0;i < Machine->drv->total_colors;i++)
 	{
-		int bit0,bit1,bit2;
+		int bit0,bit1,bit2,r,g,b;
 
 
 		/* red component */
 		bit0 = (*color_prom >> 0) & 0x01;
 		bit1 = (*color_prom >> 1) & 0x01;
 		bit2 = (*color_prom >> 2) & 0x01;
-		*(palette++) = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
+		r = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
 		/* green component */
 		bit0 = (*color_prom >> 3) & 0x01;
 		bit1 = (*color_prom >> 4) & 0x01;
 		bit2 = (*color_prom >> 5) & 0x01;
-		*(palette++) = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
+		g = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
 		/* blue component */
 		bit0 = 0;
 		bit1 = (*color_prom >> 6) & 0x01;
 		bit2 = (*color_prom >> 7) & 0x01;
-		*(palette++) = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
+		b = 0x21 * bit0 + 0x47 * bit1 + 0x97 * bit2;
 
+		palette_set_color(i,r,g,b);
 		color_prom++;
 	}
 
 	/* color_prom now points to the beginning of the lookup table */
 
+	/* TODO: there are 32 colors in the palette, but we are suing only 16 */
+	/* the only difference between the two banks is color #14, grey instead of green */
+
 	/* character lookup table */
 	/* sprites use the same color lookup table as characters */
 	for (i = 0;i < TOTAL_COLORS(0);i++)
-		COLOR(0,i) = *(color_prom++);
+		COLOR(0,i) = (*(color_prom++) & 0x0f);
 }
 
+WRITE8_HANDLER( champbas_videoram_w )
+{
+	if (videoram[offset] != data)
+	{
+		videoram[offset] = data;
+		tilemap_mark_tile_dirty(bg_tilemap, offset);
+	}
+}
 
+WRITE8_HANDLER( champbas_colorram_w )
+{
+	if (colorram[offset] != data)
+	{
+		colorram[offset] = data;
+		tilemap_mark_tile_dirty(bg_tilemap, offset);
+	}
+}
 
-/***************************************************************************
+WRITE8_HANDLER( champbas_gfxbank_w )
+{
+	if (gfxbank != (data & 0x01))
+	{
+		gfxbank = data & 0x01;
+		tilemap_mark_all_tiles_dirty(ALL_TILEMAPS);
+	}
+}
 
-  Draw the game screen in the given osd_bitmap.
-  Do NOT call osd_update_display() from this function, it will be called by
-  the main emulation engine.
+WRITE8_HANDLER( champbas_flipscreen_w )
+{
+	if (flip_screen != data)
+	{
+		flip_screen_set(data);
+		tilemap_mark_all_tiles_dirty(ALL_TILEMAPS);
+	}
+}
 
-***************************************************************************/
-void champbas_vh_screenrefresh(struct osd_bitmap *bitmap)
+static void get_bg_tile_info(int tile_index)
+{
+	int code = videoram[tile_index];
+	int color = (colorram[tile_index] & 0x1f) + 32;
+
+	SET_TILE_INFO(gfxbank, code, color, 0)
+}
+
+VIDEO_START( champbas )
+{
+	bg_tilemap = tilemap_create(get_bg_tile_info, tilemap_scan_rows, 
+		TILEMAP_OPAQUE, 8, 8, 32, 32);
+
+	if ( !bg_tilemap )
+		return 1;
+
+	return 0;
+}
+
+static void champbas_draw_sprites( struct mame_bitmap *bitmap )
 {
 	int offs;
 
-
-	/* for every character in the Video RAM, check if it has been modified */
-	/* since last time and update it accordingly. */
-	for (offs = videoram_size - 1;offs >= 0;offs--)
+	for (offs = spriteram_size - 2; offs >= 0; offs -= 2)
 	{
-		if (dirtybuffer[offs])
-		{
-			int sx,sy;
+		int code = spriteram[offs] >> 2;
+		int color = spriteram[offs + 1];
+		int flipx = spriteram[offs] & 0x01;
+		int flipy = spriteram[offs] & 0x02;
+		int sx = ((256 + 16 - spriteram_2[offs + 1]) & 0xff) - 16;
+		int sy = spriteram_2[offs] - 16;
 
-
-			dirtybuffer[offs] = 0;
-
-			sx = 8 * (offs % 32);
-			sy = 8 * (offs / 32);
-
-			drawgfx(tmpbitmap,Machine->gfx[0],
-					videoram[offs],
-					colorram[offs]+32,
-					0,0,
-					sx,sy,
-					&Machine->drv->visible_area,TRANSPARENCY_NONE,0);
-		}
-	}
-
-
-	/* copy the character mapped graphics */
-	copybitmap(bitmap,tmpbitmap,0,0,0,0,&Machine->drv->visible_area,TRANSPARENCY_NONE,0);
-
-
-	/* Draw the sprites. Note that it is important to draw them exactly in this */
-	/* order, to have the correct priorities. */
-	for (offs = spriteram_size - 2;offs > 0;offs -= 2)
-	{
-		drawgfx(bitmap,Machine->gfx[1],
-				spriteram[offs] >> 2,
-				spriteram[offs + 1],
-				spriteram[offs] & 1,spriteram[offs] & 2,
-				255 - spriteram_2[offs + 1],spriteram_2[offs] - 16,
-				&Machine->drv->visible_area,TRANSPARENCY_COLOR,0);
+		drawgfx(bitmap,
+			Machine->gfx[2 + gfxbank],
+			code, color,
+			flipx, flipy,
+			sx, sy,
+			&Machine->visible_area,
+			TRANSPARENCY_COLOR, 0);
 	}
 }
+
+VIDEO_UPDATE( champbas )
+{
+	tilemap_draw(bitmap, &Machine->visible_area, bg_tilemap, 0, 0);
+	champbas_draw_sprites(bitmap);
+}
